@@ -7,12 +7,14 @@ from pandas import DataFrame, Series
 from datashape import to_numpy, to_numpy_dtype
 from numbers import Number
 
-from ..expr import Reduction, Field, Projection, Broadcast, Selection, ndim
-from ..expr import Distinct, Sort, Head, Label, ReLabel, Expr, Slice, Join
-from ..expr import std, var, count, nunique, Summary, IsIn
-from ..expr import BinOp, UnaryOp, USub, Not, nelements, Repeat, Concat, Interp
-from ..expr import UTCFromTimestamp, DateTimeTruncate
-from ..expr import Transpose, TensorDot, Coerce
+from ..expr import (
+    Reduction, Field, Projection, Broadcast, Selection, ndim,
+    Distinct, Sort, Tail, Head, Label, ReLabel, Expr, Slice, Join,
+    std, var, count, nunique, Summary, IsIn,
+    BinOp, UnaryOp, USub, Not, nelements, Repeat, Concat, Interp,
+    UTCFromTimestamp, DateTimeTruncate,
+    Transpose, TensorDot, Coerce, isnan
+)
 from ..utils import keywords
 
 from .core import base, compute
@@ -197,9 +199,21 @@ def compute_up(t, x, **kwargs):
             keepdims=t.keepdims)
 
 
+@compute_up.register(Distinct, np.recarray)
+def recarray_distinct(t, rec, **kwargs):
+    return pd.DataFrame.from_records(rec).drop_duplicates(
+        subset=t.on or None).to_records(index=False).astype(rec.dtype)
+
+
 @dispatch(Distinct, np.ndarray)
-def compute_up(t, x, **kwargs):
-    return np.unique(x)
+def compute_up(t, arr, _recarray_distinct=recarray_distinct, **kwargs):
+    if t.on:
+        if getattr(arr.dtype, 'names', None) is not None:
+            return _recarray_distinct(t, arr, **kwargs).view(np.ndarray)
+        else:
+            raise ValueError('malformed expression: no columns to distinct on')
+
+    return np.unique(arr)
 
 
 @dispatch(Sort, np.ndarray)
@@ -223,6 +237,11 @@ def compute_up(t, x, **kwargs):
     return x[:t.n]
 
 
+@dispatch(Tail, np.ndarray)
+def compute_up(t, x, **kwargs):
+    return x[-t.n:]
+
+
 @dispatch(Label, np.ndarray)
 def compute_up(t, x, **kwargs):
     return np.array(x, dtype=[(t.label, x.dtype.type)])
@@ -318,6 +337,11 @@ def compute_up(expr, data, **kwargs):
     return result
 
 
+@dispatch(isnan, np.ndarray)
+def compute_up(expr, data, **kwargs):
+    return np.isnan(data)
+
+
 @dispatch(np.ndarray)
 def chunks(x, chunksize=1024):
     start = 0
