@@ -41,12 +41,13 @@ from ..dispatch import dispatch
 
 from .core import compute, compute_up, base
 
-from ..expr import (Projection, Field, Sort, Head, Broadcast, Selection,
+from ..expr import (Projection, Field, Sort, Head, Tail, Broadcast, Selection,
                     Reduction, Distinct, Join, By, Summary, Label, ReLabel,
                     Map, Apply, Merge, std, var, Like, Slice, summary,
                     ElemWise, DateTime, Millisecond, Expr, Symbol, IsIn,
                     UTCFromTimestamp, nelements, DateTimeTruncate, count,
-                    UnaryStringFunction, nunique, Coerce, Concat)
+                    UnaryStringFunction, nunique, Coerce, Concat,
+                    isnan, notnull)
 from ..expr import UnaryOp, BinOp, Interp
 from ..expr import symbol, common_subexpression
 
@@ -156,6 +157,16 @@ def compute_up(t, lhs, rhs, **kwargs):
     return result.reset_index()[t.fields]
 
 
+@dispatch(isnan, pd.Series)
+def compute_up(expr, data, **kwargs):
+    return data.isnull()
+
+
+@dispatch(notnull, pd.Series)
+def compute_up(expr, data, **kwargs):
+    return data.notnull()
+
+
 pandas_structure = DataFrame, Series, DataFrameGroupBy, SeriesGroupBy
 
 
@@ -192,9 +203,16 @@ def compute_up(t, s, **kwargs):
     return result
 
 
-@dispatch(Distinct, (DataFrame, Series))
+@dispatch(Distinct, DataFrame)
 def compute_up(t, df, **kwargs):
-    return df.drop_duplicates().reset_index(drop=True)
+    return df.drop_duplicates(subset=t.on or None).reset_index(drop=True)
+
+
+@dispatch(Distinct, Series)
+def compute_up(t, s, **kwargs):
+    if t.on:
+        raise ValueError('malformed expression: no columns to distinct on')
+    return s.drop_duplicates().reset_index(drop=True)
 
 
 @dispatch(nunique, DataFrame)
@@ -340,6 +358,8 @@ def compute_by(t, s, g, df):
                                   [compute(v._child, {t._child: df})
                                    for v in one.values])))
 
+    if not df.index.equals(preapply.index):
+        df = df.loc[preapply.index]
     df2 = concat_nodup(df, preapply)
 
     groups = df2.groupby(g)
@@ -433,7 +453,10 @@ def compute_up(t, df, **kwargs):
 
 @dispatch(Sort, Series)
 def compute_up(t, s, **kwargs):
-    return s.order(ascending=t.ascending)
+    try:
+        return s.sort_values(ascending=t.ascending)
+    except AttributeError:
+        return s.order(ascending=t.ascending)
 
 
 @dispatch(Head, (Series, DataFrame))
@@ -441,6 +464,11 @@ def compute_up(t, df, **kwargs):
     return df.head(t.n)
 
 
+@dispatch(Tail, (Series, DataFrame))
+def compute_up(t, df, **kwargs):
+    return df.tail(t.n)
+
+
 @dispatch(Label, DataFrame)
 def compute_up(t, df, **kwargs):
     return DataFrame(df, columns=[t.label])
@@ -573,20 +601,6 @@ def compute_up(expr, df, **kwargs):
     return df.shape[0]
 
 
-units_map = {
-    'year': 'Y',
-    'month': 'M',
-    'week': 'W',
-    'day': 'D',
-    'hour': 'h',
-    'minute': 'm',
-    'second': 's',
-    'millisecond': 'ms',
-    'microsecond': 'us',
-    'nanosecond': 'ns'
-}
-
-
 @dispatch(DateTimeTruncate, Series)
 def compute_up(expr, data, **kwargs):
     return Series(compute_up(expr, into(np.ndarray, data), **kwargs),
