@@ -8,38 +8,53 @@ SparkSQL.
 
 from __future__ import absolute_import, division, print_function
 
-from datashape.predicates import isrecord
-import sqlalchemy as sa
-from toolz import pipe, curry
+from operator import and_
+from distutils.version import LooseVersion
+
+from toolz import pipe
 from toolz.curried import filter, map
-from odo.backends.sql import dshape_to_alchemy
+
+from sqlalchemy.ext.compiler import compiles
+import sqlalchemy as sa
 
 from ..dispatch import dispatch
-from ..expr import Expr, Field, symbol
+from ..expr import Expr, symbol, Join
 from .core import compute
-from .utils import literalquery
-from .spark import Dummy
+from .utils import literalquery, istable, make_sqlalchemy_table
+from ..utils import listpack
+from .spark import jgetattr
+from pyhive.sqlalchemy_hive import HiveDialect
+from pyspark import SQLContext
 
 __all__ = []
 
+join_types = {
+    'left': 'left_outer',
+    'right': 'right_outer'
+}
 
 try:
-    from pyspark import SQLContext
-    from pyhive.sqlalchemy_hive import HiveDialect
+    from pyspark.sql import DataFrame as SparkDataFrame
 except ImportError:
-    SQLContext = Dummy
+    pass
+else:
+    @dispatch(Join, SparkDataFrame, SparkDataFrame)
+    def compute_up(t, lhs, rhs, **kwargs):
+        ands = [getattr(lhs, left) == getattr(rhs, right)
+                for left, right in zip(*map(listpack, (t.on_left, t.on_right)))]
 
+        joined = lhs.join(rhs, reduce(and_, ands), join_types.get(t.how, t.how))
 
-def make_sqlalchemy_table(expr):
-    name = expr._name
-    columns = dshape_to_alchemy(expr.dshape)
-    return sa.Table(name, sa.MetaData(), *columns)
+        prec, sec = (rhs, lhs) if t.how == 'right' else (lhs, rhs)
+        cols = [jgetattr(prec, f, jgetattr(sec, f, None)) for f in t.fields]
+        assert all(c is not None for c in cols)
+        return joined.select(*cols)
 
-
-@curry
-def istable(db, t):
-    return (isinstance(t, Field) and isrecord(t.dshape.measure) and
-            t._child.isidentical(db))
+if LooseVersion(sa.__version__) >= '1.0.0':
+    # a bug in spark sql prevents labels from being referenced properly
+    @compiles(sa.sql.elements._label_reference, 'hive')
+    def compile_label_reference(element, compiler, **kwargs):
+        return compiler.process(element.element, **kwargs)
 
 
 @dispatch(Expr, SQLContext)
