@@ -19,7 +19,7 @@ from __future__ import absolute_import, division, print_function
 import itertools
 from itertools import chain
 
-from operator import and_, eq
+from operator import and_, eq, attrgetter
 from copy import copy
 
 import sqlalchemy as sa
@@ -27,12 +27,14 @@ import sqlalchemy as sa
 from sqlalchemy import sql, Table, MetaData
 from sqlalchemy.sql import Selectable, Select, functions as safuncs
 from sqlalchemy.ext.compiler import compiles
-from sqlalchemy.sql.elements import ClauseElement, ColumnElement
+from sqlalchemy.sql.elements import ClauseElement, ColumnElement, ColumnClause
+from sqlalchemy.sql.selectable import FromClause, ScalarSelect
 from sqlalchemy.engine import Engine
 
 import toolz
 
-from toolz import unique, concat, pipe
+from toolz import unique, concat, pipe, first
+from toolz.compatibility import zip
 from toolz.curried import map
 
 import numpy as np
@@ -44,15 +46,19 @@ from multipledispatch import MDNotImplementedError
 
 from odo.backends.sql import metadata_of_engine, dshape_to_alchemy
 
+from datashape.predicates import iscollection, isscalar, isrecord
+
 from ..dispatch import dispatch
 
 from .core import compute_up, compute, base
 
-from ..expr import Projection, Selection, Field, Broadcast, Expr, IsIn, Slice
-from ..expr import (BinOp, UnaryOp, Join, mean, var, std, Reduction, count,
-                    FloorDiv, UnaryStringFunction, strlen, DateTime, Coerce)
-from ..expr import nunique, Distinct, By, Sort, Head, Label, ReLabel, Merge
-from ..expr import common_subexpression, Summary, Like, nelements, Concat
+
+from ..expr import (
+    Projection, Selection, Field, Broadcast, Expr, IsIn, Slice, BinOp, UnaryOp,
+    Join, mean, var, std, Reduction, count, FloorDiv, UnaryStringFunction,
+    strlen, DateTime, Coerce, nunique, Distinct, By, Sort, Head, Label, Concat,
+    ReLabel, Merge, common_subexpression, Summary, Like, nelements, notnull
+)
 
 from ..expr.broadcast import broadcast_collect
 from ..expr.math import isnan
@@ -96,8 +102,7 @@ def compute_up(t, s, **kwargs):
     cols = list(inner_columns(s))
     d = dict((t._scalars[0][c], cols[i])
              for i, c in enumerate(t._scalars[0].fields))
-    name = t._scalar_expr._name
-    result = compute(t._scalar_expr, d, post_compute=False).label(name)
+    result = compute(t._scalar_expr, d, post_compute=False).label(t._name)
 
     s = copy(s)
     s.append_column(result)
@@ -109,8 +114,7 @@ def compute_up(t, s, **kwargs):
     cols = list(inner_columns(s))
     d = dict((t._scalars[0][c], cols[i])
              for i, c in enumerate(t._scalars[0].fields))
-    name = t._scalar_expr._name
-    return compute(t._scalar_expr, d, post_compute=False).label(name)
+    return compute(t._scalar_expr, d, post_compute=False).label(t._name)
 
 
 @dispatch(Concat, (Select, Selectable), (Select, Selectable))
@@ -138,6 +142,17 @@ def compute_up(t, data, **kwargs):
         return t.op(t.lhs, data)
 
 
+@dispatch(BinOp, Select)
+def compute_up(t, data, **kwargs):
+    assert len(data.c) == 1, \
+        'Select cannot have more than a single column when doing arithmetic'
+    column = first(data.inner_columns)
+    if isinstance(t.lhs, Expr):
+        return t.op(column, t.rhs)
+    else:
+        return t.op(t.lhs, column)
+
+
 @compute_up.register(BinOp, (ColumnElement, base), ColumnElement)
 @compute_up.register(BinOp, ColumnElement, (ColumnElement, base))
 def binop_sql(t, lhs, rhs, **kwargs):
@@ -169,23 +184,18 @@ def compute_up(t, s, **kwargs):
     return getattr(t, 'op', getattr(safuncs, sym, getattr(sa.func, sym)))(s)
 
 
-@dispatch(Selection, Select)
-def compute_up(t, s, scope=None, **kwargs):
-    ns = dict((t._child[col.name], col) for col in s.inner_columns)
-    predicate = compute(t.predicate, toolz.merge(ns, scope),
-                        optimize=False, post_compute=False)
-    if isinstance(predicate, Select):
-        predicate = list(list(predicate.columns)[0].base_columns)[0]
-    return s.where(predicate)
+@dispatch(Selection, sa.sql.ColumnElement)
+def compute_up(expr, data, scope=None, **kwargs):
+    predicate = compute(expr.predicate, data, post_compute=False)
+    return sa.select([data]).where(predicate)
 
 
 @dispatch(Selection, Selectable)
 def compute_up(t, s, scope=None, **kwargs):
-    ns = dict((t._child[col.name], lower_column(col)) for col in s.columns)
+    ns = dict((t._child[col.name], col)
+              for col in getattr(s, 'inner_columns', s.columns))
     predicate = compute(t.predicate, toolz.merge(ns, scope),
                         optimize=False, post_compute=False)
-    if isinstance(predicate, Select):
-        predicate = list(list(predicate.columns)[0].base_columns)[0]
     try:
         return s.where(predicate)
     except AttributeError:
@@ -206,9 +216,6 @@ def select(s):
     return s
 
 
-def computefull(t, s):
-    return select(compute(t, s))
-
 table_names = ('table_%d' % i for i in itertools.count(1))
 
 
@@ -244,11 +251,22 @@ def _join_selectables(a, b, condition=None, **kwargs):
                                 a.join(b.froms[0], condition, **kwargs))
 
 
+
 @dispatch(ClauseElement, ClauseElement)
 def _join_selectables(a, b, condition=None, **kwargs):
     return a.join(b, condition, **kwargs)
 
 
+_getname = attrgetter('name')
+
+
+def _clean_join_name(opposite_side_colnames, suffix, c):
+    if c.name not in opposite_side_colnames:
+        return c
+    else:
+        return c.label(c.name + suffix)
+
+
 @dispatch(Join, ClauseElement, ClauseElement)
 def compute_up(t, lhs, rhs, **kwargs):
     if isinstance(lhs, ColumnElement):
@@ -282,13 +300,13 @@ def compute_up(t, lhs, rhs, **kwargs):
     # Perform join
     if t.how == 'inner':
         join = _join_selectables(lhs, rhs, condition=condition)
-        main, other = lhs, rhs
+        main = lhs
     elif t.how == 'left':
         main, other = lhs, rhs
         join = _join_selectables(lhs, rhs, condition=condition, isouter=True)
     elif t.how == 'right':
         join = _join_selectables(rhs, lhs, condition=condition, isouter=True)
-        main, other = rhs, lhs
+        main = rhs
     else:
         # http://stackoverflow.com/questions/20361017/sqlalchemy-full-outer-join
         raise ValueError("SQLAlchemy doesn't support full outer Join")
@@ -307,18 +325,21 @@ def compute_up(t, lhs, rhs, **kwargs):
         cols = lambda x: list(x.columns)
 
     main_cols = cols(main)
-    other_cols = cols(other)
     left_cols = cols(lhs)
+    left_names = set(map(_getname, left_cols))
     right_cols = cols(rhs)
+    right_names = set(map(_getname, right_cols))
 
     left_suffix, right_suffix = t.suffixes
     fields = [
         f.replace(left_suffix, '').replace(right_suffix, '') for f in t.fields
     ]
     columns = [c for c in main_cols if c.name in t._on_left]
-    columns += [c for c in left_cols
+    columns += [_clean_join_name(right_names, left_suffix, c)
+                for c in left_cols
                 if c.name in fields and c.name not in t._on_left]
-    columns += [c for c in right_cols
+    columns += [_clean_join_name(left_names, right_suffix, c)
+                for c in right_cols
                 if c.name in fields and c.name not in t._on_right]
 
     if isinstance(join, Select):
@@ -332,6 +353,31 @@ names = {
 }
 
 
+def reconstruct_select(columns, original, **kwargs):
+    return sa.select(columns,
+                     from_obj=kwargs.pop('from_obj', None),
+                     whereclause=kwargs.pop('whereclause',
+                                            getattr(original,
+                                                    '_whereclause', None)),
+                     bind=kwargs.pop('bind', original.bind),
+                     distinct=kwargs.pop('distinct',
+                                         getattr(original,
+                                                 '_distinct', False)),
+                     group_by=kwargs.pop('group_by',
+                                         getattr(original,
+                                                 '_group_by_clause', None)),
+                     having=kwargs.pop('having',
+                                       getattr(original, '_having', None)),
+                     limit=kwargs.pop('limit',
+                                      getattr(original, '_limit', None)),
+                     offset=kwargs.pop('offset',
+                                       getattr(original, '_offset', None)),
+                     order_by=kwargs.pop('order_by',
+                                         getattr(original,
+                                                 '_order_by_clause', None)),
+                     **kwargs)
+
+
 @dispatch((nunique, Reduction), Select)
 def compute_up(expr, data, **kwargs):
     if expr.axis != (0,):
@@ -343,17 +389,22 @@ def compute_up(expr, data, **kwargs):
     return select([compute(expr, d, post_compute=False)])
 
 
-@dispatch(Distinct, sa.Column)
+@dispatch(Distinct, ColumnElement)
 def compute_up(t, s, **kwargs):
-    return s.distinct().label(t._name)
+    return s.distinct(*t.on).label(t._name)
 
 
 @dispatch(Distinct, Select)
 def compute_up(t, s, **kwargs):
-    return s.distinct()
+    return s.distinct(*t.on)
 
 
-@dispatch(Reduction, sql.elements.ClauseElement)
+@dispatch(Distinct, Selectable)
+def compute_up(t, s, **kwargs):
+    return select(s).distinct(*t.on)
+
+
+@dispatch(Reduction, ClauseElement)
 def compute_up(t, s, **kwargs):
     if t.axis != (0,):
         raise ValueError('axis not equal to 0 not defined for SQL reductions')
@@ -381,7 +432,7 @@ def compute_up(t, s, **kwargs):
 
 @dispatch(count, Selectable)
 def compute_up(t, s, **kwargs):
-    return compute_up(t, select(s), **kwargs)
+    return s.count()
 
 
 @dispatch(count, sa.Table)
@@ -430,20 +481,14 @@ def compute_up(expr, data, **kwargs):
     return select(data).distinct().alias(next(aliases)).count()
 
 
-@dispatch(Distinct, Selectable)
-def compute_up(t, s, **kwargs):
-    return select(s).distinct()
-
-
 @dispatch(By, sa.Column)
-def compute_up(t, s, **kwargs):
-    grouper = lower_column(s)
-    scope = {t._child: s}
-    app = t.apply
+def compute_up(expr, data, **kwargs):
+    grouper = lower_column(data)
+    app = expr.apply
     if isinstance(app, Reduction):
-        reductions = [compute(app, scope, post_compute=False)]
+        reductions = [compute(app, data, post_compute=False)]
     elif isinstance(app, Summary):
-        reductions = [compute(val, scope, post_compute=None).label(name)
+        reductions = [compute(val, data, post_compute=None).label(name)
                       for val, name in zip(app.values, app.fields)]
 
     return sa.select([grouper] + reductions).group_by(grouper)
@@ -451,22 +496,17 @@ def compute_up(t, s, **kwargs):
 
 @dispatch(By, ClauseElement)
 def compute_up(expr, data, **kwargs):
-    if (isinstance(expr.grouper, (Field, Projection)) or
-            expr.grouper is expr._child):
-        grouper = [lower_column(data.c.get(col))
-                   for col in expr.grouper.fields]
-    elif isinstance(expr.grouper, DateTime):
-        grouper = [compute(expr.grouper, data, post_compute=False)]
-    else:
-        raise ValueError("Grouper must be a projection, field or "
-                         "DateTime expression, got %s" % expr.grouper)
+    if not valid_grouper(expr):
+        raise TypeError("Grouper must have a non-nested record or one "
+                        "dimensional collection datashape, "
+                        "got %s of type %r with dshape %s" %
+                        (expr.grouper, type(expr.grouper).__name__,
+                         expr.dshape))
+    grouper = get_inner_columns(compute(expr.grouper, data,
+                                        post_compute=False))
     app = expr.apply
-    scope = {expr._child: data}
-    if isinstance(expr.apply, Reduction):
-        reductions = [compute(app, scope, post_compute=False)]
-    elif isinstance(expr.apply, Summary):
-        reductions = [compute(val, scope, post_compute=False).label(name)
-                      for val, name in zip(app.values, app.fields)]
+    reductions = [compute(val, data, post_compute=False).label(name)
+                  for val, name in zip(app.values, app.fields)]
 
     return sa.select(grouper + reductions).group_by(*grouper)
 
@@ -519,45 +559,71 @@ def alias_it(s):
         return s
 
 
+def is_nested_record(measure):
+    """Predicate for checking whether `measure` is a nested ``Record`` dshape
+
+    Examples
+    --------
+    >>> from datashape import dshape
+    >>> is_nested_record(dshape('{a: int32, b: int32}').measure)
+    False
+    >>> is_nested_record(dshape('{a: var * ?float64, b: ?string}').measure)
+    True
+    """
+    if not isrecord(measure):
+        raise TypeError('Input must be a Record type got %s of type %r' %
+                        (measure, type(measure).__name__))
+    return not all(isscalar(t) for t in measure.types)
+
+
+def valid_grouper(expr):
+    ds = expr.dshape
+    measure = ds.measure
+    return (iscollection(ds) and
+            (isscalar(measure) or
+             (isrecord(measure) and not is_nested_record(measure))))
+
+
+def valid_reducer(expr):
+    ds = expr.dshape
+    measure = ds.measure
+    return (not iscollection(ds) and
+            (isscalar(measure) or
+             (isrecord(measure) and not is_nested_record(measure))))
+
+
 @dispatch(By, Select)
-def compute_up(t, s, **kwargs):
+def compute_up(expr, data, **kwargs):
+    if not valid_grouper(expr):
+        raise TypeError("Grouper must have a non-nested record or one "
+                        "dimensional collection datashape, "
+                        "got %s of type %r with dshape %s" %
+                        (expr.grouper, type(expr.grouper).__name__, expr.dshape))
 
-    # TODO: this should restrict according to dshape, not expression type
-    if not (isinstance(t.grouper, (Field, Projection, DateTime)) or
-            t.grouper is t._child):
-        raise ValueError("Grouper must be a Field, Projection or DateTime, "
-                         "got %s of type %r" % (t.grouper,
-                                                type(t.grouper).__name__))
+    s = alias_it(data)
 
-    s = alias_it(s)
-
-    # TODO: Do we need to be this restrictive here?
-    if isinstance(t.apply, (Summary, Reduction)):
-        reduction = compute(t.apply, {t._child: s}, post_compute=False)
+    if valid_reducer(expr.apply):
+        reduction = compute(expr.apply, s, post_compute=False)
     else:
-        raise TypeError('apply must be a Summary or Reduction expression')
+        raise TypeError('apply must be a Summary expression')
 
-    grouper = get_inner_columns(compute(t.grouper, {t._child: s}))
-    reduction_columns = pipe(reduction.inner_columns, map(get_inner_columns),
+    grouper = get_inner_columns(compute(expr.grouper, s, post_compute=False))
+    reduction_columns = pipe(reduction.inner_columns,
+                             map(get_inner_columns),
                              concat)
     columns = list(unique(chain(grouper, reduction_columns)))
     if (not isinstance(s, sa.sql.selectable.Alias) or
             (hasattr(s, 'froms') and isinstance(s.froms[0],
                                                 sa.sql.selectable.Join))):
         assert len(s.froms) == 1, 'only a single FROM clause supported for now'
-        from_obj = s.froms[0]
+        from_obj, = s.froms
     else:
         from_obj = None
 
-    return sa.select(columns=columns,
-                     whereclause=getattr(s, 'element', s)._whereclause,
-                     from_obj=from_obj,
-                     distinct=getattr(s, 'element', s)._distinct,
-                     group_by=grouper,
-                     having=None,  # TODO: we don't have an expression for this
-                     limit=getattr(s, 'element', s)._limit,
-                     offset=getattr(s, 'element', s)._offset,
-                     order_by=get_clause(getattr(s, 'element', s), 'order_by'))
+    return reconstruct_select(columns,
+                              getattr(s, 'element', s),
+                              from_obj=from_obj,
+                              group_by=grouper)
 
 
 @dispatch(Sort, (Selectable, Select))
@@ -576,34 +642,58 @@ def compute_up(t, s, **kwargs):
     return s.order_by(*cols)
 
 
-@dispatch(Head, Select)
+@dispatch(Head, FromClause)
 def compute_up(t, s, **kwargs):
     if s._limit is not None and s._limit <= t.n:
         return s
     return s.limit(t.n)
 
 
-@dispatch(Head, ClauseElement)
+@dispatch(Head, sa.Table)
 def compute_up(t, s, **kwargs):
-    return select(s).limit(t.n)
+    return s.select().limit(t.n)
 
 
-@dispatch(Label, ClauseElement)
+@dispatch(Head, ColumnElement)
+def compute_up(t, s, **kwargs):
+    return sa.select([s]).limit(t.n)
+
+
+@dispatch(Head, ScalarSelect)
+def compute_up(t, s, **kwargs):
+    return compute(t, s.element, post_compute=False)
+
+
+@dispatch(Label, ColumnElement)
 def compute_up(t, s, **kwargs):
     return s.label(t.label)
 
 
-@dispatch(ReLabel, Selectable)
+@dispatch(Label, FromClause)
 def compute_up(t, s, **kwargs):
-    columns = [getattr(s.c, col).label(new_col)
-               if col != new_col else
-               getattr(s.c, col)
-               for col, new_col in zip(t._child.fields, t.fields)]
-
-    return select(columns)
+    assert len(s.c) == 1, \
+        'expected %s to have a single column but has %d' % (s, len(s.c))
+    inner_column, = s.inner_columns
+    return reconstruct_select([inner_column.label(t.label)], s).as_scalar()
 
 
-@dispatch(sa.sql.FromClause)
+@dispatch(Expr, ScalarSelect)
+def post_compute(t, s, **kwargs):
+    return s.element
+
+
+@dispatch(ReLabel, Selectable)
+def compute_up(expr, data, **kwargs):
+    names = data.c.keys()
+    assert names == expr._child.fields
+    d = dict(zip(names, getattr(data, 'inner_columns', data.c)))
+    return sa.select(
+        d[col].label(new_col) if col != new_col else d[col]
+        for col, new_col in zip(expr._child.fields, expr.fields)
+    )
+
+
+@dispatch(FromClause)
 def get_inner_columns(sel):
     try:
         return list(sel.inner_columns)
@@ -611,16 +701,16 @@ def get_inner_columns(sel):
         return list(map(lower_column, sel.c.values()))
 
 
-@dispatch(sa.sql.elements.ColumnElement)
+@dispatch(ColumnElement)
 def get_inner_columns(c):
     return [c]
 
 
-@dispatch(sa.sql.selectable.ScalarSelect)
+@dispatch(ScalarSelect)
 def get_inner_columns(sel):
     inner_columns = list(sel.inner_columns)
     assert len(inner_columns) == 1, 'ScalarSelect should have only ONE column'
-    return list(map(lower_column, sel.inner_columns))
+    return list(map(lower_column, inner_columns))
 
 
 @dispatch(sa.sql.functions.Function)
@@ -655,7 +745,7 @@ def get_all_froms(t):
     return [t]
 
 
-@dispatch(sa.sql.elements.ColumnClause)
+@dispatch(ColumnClause)
 def get_all_froms(c):
     return [c.table]
 
@@ -684,15 +774,7 @@ def compute_up(expr, data, **kwargs):
     # we need these getattrs if data is a ColumnClause or Table
     from_obj = get_all_froms(data)
     assert len(from_obj) == 1, 'only a single FROM clause supported'
-    return sa.select(columns,
-                     from_obj=from_obj,
-                     whereclause=getattr(data, '_whereclause', None),
-                     distinct=getattr(data, '_distinct', False),
-                     having=getattr(data, '_having', None),
-                     group_by=get_clause(data, 'group_by'),
-                     limit=getattr(data, '_limit', None),
-                     offset=getattr(data, '_offset', None),
-                     order_by=get_clause(data, 'order_by'))
+    return reconstruct_select(columns, data, from_obj=from_obj)
 
 
 @dispatch(Summary, Select)
@@ -712,8 +794,11 @@ def compute_up(t, s, scope=None, **kwargs):
 
 @dispatch(Summary, ClauseElement)
 def compute_up(t, s, **kwargs):
-    return select([compute(value, {t._child: s}, post_compute=None).label(name)
-                   for value, name in zip(t.values, t.fields)])
+    scope = {t._child: s}
+    return sa.select(
+        compute(value, scope, post_compute=None).label(name)
+        for value, name in zip(t.values, t.fields)
+    )
 
 
 @dispatch(Like, Selectable)
@@ -757,6 +842,11 @@ def compute_up(expr, data, **kwargs):
     return getattr(sa.sql.func, func_name)(data).label(expr._name)
 
 
+@dispatch(notnull, ColumnElement)
+def compute_up(expr, data, **kwargs):
+    return data != None
+
+
 @toolz.memoize
 def table_of_metadata(metadata, name):
     if metadata.schema is not None:
@@ -807,7 +897,7 @@ def engine_of(x):
     raise NotImplementedError("Can't deterimine engine of %s" % x)
 
 
-@dispatch(Expr, sa.sql.elements.ClauseElement)
+@dispatch(Expr, ClauseElement)
 def optimize(expr, _):
     return broadcast_collect(expr)
 
@@ -817,7 +907,7 @@ def compute_up(expr, data, **kwargs):
     return table_of_metadata(data, expr._name)
 
 
-@dispatch(Expr, sa.sql.elements.ClauseElement)
+@dispatch(Expr, ClauseElement)
 def post_compute(_, s, **kwargs):
     return select(s)
 
