@@ -4,4 +4,4 @@ import pytest
 @pytest.fixture(scope='session')
 def sc():
     pyspark = pytest.importorskip('pyspark')
-    return pyspark.SparkContext('local', 'blaze-test')
+    return pyspark.SparkContext('local', 'blaze')
