@@ -104,7 +104,7 @@ def test_count_nan():
     assert compute(t.count(), x) == 2
 
 
-def test_Distinct():
+def test_distinct():
     x = np.array([('Alice', 100),
                   ('Alice', -200),
                   ('Bob', 100),
@@ -119,6 +119,62 @@ def test_Distinct():
               np.unique(x))
 
 
+def test_distinct_on_recarray():
+    rec = pd.DataFrame(
+        [[0, 1],
+         [0, 2],
+         [1, 1],
+         [1, 2]],
+        columns=('a', 'b'),
+    ).to_records(index=False)
+
+    s = symbol('s', discover(rec))
+    assert (
+        compute(s.distinct('a'), rec) ==
+        pd.DataFrame(
+            [[0, 1],
+             [1, 1]],
+            columns=('a', 'b'),
+        ).to_records(index=False)
+    ).all()
+
+
+def test_distinct_on_structured_array():
+    arr = np.array(
+        [(0., 1.),
+         (0., 2.),
+         (1., 1.),
+         (1., 2.)],
+        dtype=[('a', 'f4'), ('b', 'f4')],
+    )
+
+    s = symbol('s', discover(arr))
+    assert(
+        compute(s.distinct('a'), arr) ==
+        np.array([(0., 1.), (1., 1.)], dtype=arr.dtype)
+    ).all()
+
+
+def test_distinct_on_str():
+    rec = pd.DataFrame(
+        [['a', 'a'],
+         ['a', 'b'],
+         ['b', 'a'],
+         ['b', 'b']],
+        columns=('a', 'b'),
+    ).to_records(index=False).astype([('a', '<U1'), ('b', '<U1')])
+
+    s = symbol('s', discover(rec))
+    assert (
+        compute(s.distinct('a'), rec) ==
+        pd.DataFrame(
+            [['a', 'a'],
+             ['b', 'a']],
+            columns=('a', 'b'),
+        ).to_records(index=False).astype([('a', '<U1'), ('b', '<U1')])
+    ).all()
+
+
 def test_sort():
     assert eq(compute(t.sort('amount'), x),
               np.sort(x, order='amount'))
@@ -138,6 +194,11 @@ def test_head():
               x[:2])
 
 
+def test_tail():
+    assert eq(compute(t.tail(2), x),
+              x[-2:])
+
+
 def test_label():
     expected = x['amount'] * 10
     expected = np.array(expected, dtype=[('foo', 'i8')])
@@ -352,7 +413,7 @@ def test_mixed_types():
                  dtype=[('count', '<i4'), ('total', '<i8')])
     aggregate = symbol('aggregate', discover(x))
     result = compute(aggregate.total.sum(axis=(0,)) /
-                     aggregate.count.sum(axis=(0,)), x)
+                     aggregate['count'].sum(axis=(0,)), x)
     expected = (x['total'].sum(axis=0, keepdims=True) /
                 x['count'].sum(axis=0, keepdims=True)).squeeze()
     np.testing.assert_array_equal(result, expected)
