@@ -43,13 +43,6 @@ Not
 '''.split()
 
 
-def name(o):
-    if hasattr(o, '_name'):
-        return o._name
-    else:
-        return None
-
-
 class BinOp(ElemWise):
     __slots__ = '_hash', 'lhs', 'rhs'
     __inputs__ = 'lhs', 'rhs'
@@ -67,13 +60,16 @@ class BinOp(ElemWise):
     def _name(self):
         if not isscalar(self.dshape.measure):
             return None
-        l, r = name(self.lhs), name(self.rhs)
-        if l and not r:
+        l = getattr(self.lhs, '_name', None)
+        r = getattr(self.rhs, '_name', None)
+        if l is not None and r is None:
             return l
-        if r and not l:
+        elif r is not None and l is None:
             return r
-        if l == r:
+        elif l == r:
             return l
+        else:
+            return None
 
     @property
     def _inputs(self):
