@@ -1,21 +1,30 @@
 from __future__ import absolute_import, division, print_function
 
-from toolz import (
-    isdistinct, frequencies, concat as tconcat, unique, get, first,
-)
+from functools import partial
+from itertools import chain
+
 import datashape
-from datashape import DataShape, Option, Record, Unit, dshape, var, Fixed, Var
+from datashape import (
+    DataShape, Option, Record, Unit, dshape, var, Fixed, Var, promote, object_,
+)
 from datashape.predicates import isscalar, iscollection, isrecord
+from toolz import (
+    isdistinct, frequencies, concat as tconcat, unique, get, first, compose,
+    keymap,
+)
+import toolz.curried.operator as op
+from odo.utils import copydoc
 
 from .core import common_subexpression
-from .expressions import Expr, ElemWise, label
+from .expressions import Expr, ElemWise, label, Field
 from .expressions import dshape_method_list
-from ..compatibility import zip_longest
+from ..compatibility import zip_longest, _strtypes
+from ..utils import listpack
 
 
 __all__ = ['Sort', 'Distinct', 'Head', 'Merge', 'IsIn', 'isin', 'distinct',
            'merge', 'head', 'sort', 'Join', 'join', 'transform', 'Concat',
-           'concat']
+           'concat', 'Tail', 'tail']
 
 
 class Sort(Expr):
@@ -86,6 +95,11 @@ class Distinct(Expr):
 
     """ Remove duplicate elements from an expression
 
+    Parameters
+    ----------
+    on : tuple of :class:`~blaze.expr.expressions.Field`
+        The subset of fields or names of fields to be distinct on.
+
     Examples
     --------
     >>> from blaze import symbol
@@ -99,8 +113,24 @@ class Distinct(Expr):
     >>> from blaze.compute.python import compute
     >>> sorted(compute(e, data))
     [('Alice', 100, 1), ('Bob', 200, 2)]
+
+    Use a subset by passing `on`:
+
+    >>> import pandas as pd
+    >>> e = distinct(t, 'name')
+    >>> data = pd.DataFrame([['Alice', 100, 1],
+    ...                      ['Alice', 200, 2],
+    ...                      ['Bob', 100, 1],
+    ...                      ['Bob', 200, 2]],
+    ...                     columns=['name', 'amount', 'id'])
+    >>> compute(e, data)
+        name  amount  id
+    0  Alice     100   1
+    1    Bob     100   1
+
+
     """
-    __slots__ = '_hash', '_child',
+    __slots__ = '_hash', '_child', 'on'
 
     @property
     def dshape(self):
@@ -115,27 +145,33 @@ class Distinct(Expr):
         return self._child._name
 
     def __str__(self):
-        return 'distinct(%s)' % self._child
+        return 'distinct({child}{on})'.format(
+            child=self._child,
+            on=(', ' if self.on else '') + ', '.join(map(str, self.on))
+        )
 
 
-def distinct(expr):
-    return Distinct(expr)
+@copydoc(Distinct)
+def distinct(expr, *on):
+    fields = frozenset(expr.fields)
+    _on = []
+    append = _on.append
+    for n in on:
+        if isinstance(n, Field):
+            if n._child.isidentical(expr):
+                n = n._name
+            else:
+                raise ValueError('{0} is not a field of {1}'.format(n, expr))
+        if not isinstance(n, _strtypes):
+            raise TypeError('on must be a name or field, not: {0}'.format(n))
+        elif n not in fields:
+            raise ValueError('{0} is not a field of {1}'.format(n, expr))
+        append(n)
+
+    return Distinct(expr, tuple(_on))
 
 
-distinct.__doc__ = Distinct.__doc__
-
-
-class Head(Expr):
-
-    """ First `n` elements of collection
-
-    Examples
-    --------
-    >>> from blaze import symbol
-    >>> accounts = symbol('accounts', 'var * {name: string, amount: int}')
-    >>> accounts.head(5).dshape
-    dshape("5 * {name: string, amount: int32}")
-    """
+class _HeadOrTail(Expr):
     __slots__ = '_hash', '_child', 'n'
 
     @property
@@ -150,36 +186,54 @@ class Head(Expr):
         return self._child._name
 
     def __str__(self):
-        return '%s.head(%d)' % (self._child, self.n)
+        return '%s.%s(%d)' % (self._child, type(self).__name__.lower(), self.n)
 
 
+class Head(_HeadOrTail):
+
+    """ First `n` elements of collection
+
+    Examples
+    --------
+    >>> from blaze import symbol
+    >>> accounts = symbol('accounts', 'var * {name: string, amount: int}')
+    >>> accounts.head(5).dshape
+    dshape("5 * {name: string, amount: int32}")
+
+    See Also
+    --------
+
+    blaze.expr.collections.Tail
+    """
+    pass
+
+
+@copydoc(Head)
 def head(child, n=10):
     return Head(child, n)
 
-head.__doc__ = Head.__doc__
+
+class Tail(_HeadOrTail):
+    """ Last `n` elements of collection
+
+    Examples
+    --------
+    >>> from blaze import symbol
+    >>> accounts = symbol('accounts', 'var * {name: string, amount: int}')
+    >>> accounts.tail(5).dshape
+    dshape("5 * {name: string, amount: int32}")
+
+    See Also
+    --------
+
+    blaze.expr.collections.Head
+    """
+    pass
 
 
-def merge(*exprs, **kwargs):
-    if len(exprs) + len(kwargs) == 1:
-        if exprs:
-            return exprs[0]
-        if kwargs:
-            [(k, v)] = kwargs.items()
-            return v.label(k)
-    # Get common sub expression
-    exprs += tuple(label(v, k) for k, v in sorted(kwargs.items(), key=first))
-    try:
-        child = common_subexpression(*exprs)
-    except Exception:
-        raise ValueError("No common subexpression found for input expressions")
-
-    result = Merge(child, exprs)
-
-    if not isdistinct(result.fields):
-        raise ValueError("Repeated columns found: " + ', '.join(k for k, v in
-                                                                frequencies(result.fields).items() if v > 1))
-
-    return result
+@copydoc(Tail)
+def tail(child, n=10):
+    return Tail(child, n)
 
 
 def transform(t, replace=True, **kwargs):
@@ -263,7 +317,31 @@ class Merge(ElemWise):
         return list(unique(tconcat(i._leaves() for i in self.children)))
 
 
-merge.__doc__ = Merge.__doc__
+@copydoc(Merge)
+def merge(*exprs, **kwargs):
+    if len(exprs) + len(kwargs) == 1:
+        if exprs:
+            return exprs[0]
+        if kwargs:
+            [(k, v)] = kwargs.items()
+            return v.label(k)
+    # Get common sub expression
+    exprs += tuple(label(v, k) for k, v in sorted(kwargs.items(), key=first))
+    try:
+        child = common_subexpression(*exprs)
+    except Exception:
+        raise ValueError("No common subexpression found for input expressions")
+
+    result = Merge(child, exprs)
+
+    if not isdistinct(result.fields):
+        raise ValueError(
+            "Repeated columns found: " + ', '.join(
+                k for k, v in frequencies(result.fields).items() if v > 1
+            ),
+        )
+
+    return result
 
 
 def unpack(l):
@@ -288,9 +366,17 @@ class Join(Expr):
     ----------
     lhs, rhs : Expr
         Expressions to join
-    on_left : string
-    on_right : string
-    suffixes: pair of strings
+    on_left : str, optional
+        The fields from the left side to join on.
+        If no ``on_right`` is passed, then these are the fields for both
+        sides.
+    on_right : str, optional
+        The fields from the right side to join on.
+    how : {'inner', 'outer', 'left', 'right'}
+        What type of join to perform.
+    suffixes: pair of str
+        The suffixes to be applied to the left and right sides
+        in order to resolve duplicate field names.
 
     Examples
     --------
@@ -319,17 +405,17 @@ class Join(Expr):
 
     @property
     def on_left(self):
-        if isinstance(self._on_left, tuple):
-            return list(self._on_left)
-        else:
-            return self._on_left
+        on_left = self._on_left
+        if isinstance(on_left, tuple):
+            return list(on_left)
+        return on_left
 
     @property
     def on_right(self):
-        if isinstance(self._on_right, tuple):
-            return list(self._on_right)
-        else:
-            return self._on_right
+        on_right = self._on_right
+        if isinstance(on_right, tuple):
+            return list(on_right)
+        return on_right
 
     @property
     def schema(self):
@@ -356,35 +442,57 @@ class Join(Expr):
         """
         option = lambda dt: dt if isinstance(dt, Option) else Option(dt)
 
-        joined = [[name, dt] for name, dt in self.lhs.schema[0].parameters[0]
-                  if name in self.on_left]
+        on_left = self.on_left
+        if not isinstance(on_left, list):
+            on_left = on_left,
 
-        left = [[name, dt] for name, dt in
-                zip(self.lhs.fields, types_of_fields(
-                    self.lhs.fields, self.lhs))
-                if name not in self.on_left]
+        on_right = self.on_right
+        if not isinstance(on_right, list):
+            on_right = on_right,
 
-        right = [[name, dt] for name, dt in
-                 zip(self.rhs.fields, types_of_fields(
-                     self.rhs.fields, self.rhs))
-                 if name not in self.on_right]
+        right_types = keymap(
+            dict(zip(on_right, on_left)).get,
+            self.rhs.dshape.measure.dict,
+        )
+        joined = (
+            (name, promote(dt, right_types[name], promote_option=False))
+            for n, (name, dt) in enumerate(filter(
+                compose(op.contains(on_left), first),
+                self.lhs.dshape.measure.fields,
+            ))
+        )
+
+        left = [
+            (name, dt) for name, dt in zip(
+                self.lhs.fields,
+                types_of_fields(self.lhs.fields, self.lhs)
+            ) if name not in on_left
+        ]
+
+        right = [
+            (name, dt) for name, dt in zip(
+                self.rhs.fields,
+                types_of_fields(self.rhs.fields, self.rhs)
+            ) if name not in on_right
+        ]
 
         # Handle overlapping but non-joined case, e.g.
-        left_other = [name for name, dt in left if name not in self.on_left]
-        right_other = [name for name, dt in right if name not in self.on_right]
-        overlap = set.intersection(set(left_other), set(right_other))
+        left_other = set(name for name, dt in left if name not in on_left)
+        right_other = set(name for name, dt in right if name not in on_right)
+        overlap = left_other & right_other
+
         left_suffix, right_suffix = self.suffixes
-        left = [[name + left_suffix if name in overlap else name, dt]
-                for name, dt in left]
-        right = [[name + right_suffix if name in overlap else name, dt]
-                 for name, dt in right]
+        left = ((name + left_suffix if name in overlap else name, dt)
+                for name, dt in left)
+        right = ((name + right_suffix if name in overlap else name, dt)
+                 for name, dt in right)
 
         if self.how in ('right', 'outer'):
-            left = [[name, option(dt)] for name, dt in left]
+            left = ((name, option(dt)) for name, dt in left)
         if self.how in ('left', 'outer'):
-            right = [[name, option(dt)] for name, dt in right]
+            right = ((name, option(dt)) for name, dt in right)
 
-        return dshape(Record(joined + left + right))
+        return dshape(Record(chain(joined, left, right)))
 
     @property
     def dshape(self):
@@ -413,11 +521,12 @@ def types_of_fields(fields, expr):
     else:
         if isinstance(fields, (tuple, list, set)):
             assert len(fields) == 1
-            fields = fields[0]
+            fields, = fields
         assert fields == expr._name
         return expr.dshape.measure
 
 
+@copydoc(Join)
 def join(lhs, rhs, on_left=None, on_right=None,
          how='inner', suffixes=('_left', '_right')):
     if not on_left and not on_right:
@@ -431,10 +540,28 @@ def join(lhs, rhs, on_left=None, on_right=None,
     if isinstance(on_right, tuple):
         on_right = list(on_right)
     if not on_left or not on_right:
-        raise ValueError("Can not Join.  No shared columns between %s and %s" %
-                         (lhs, rhs))
-    if types_of_fields(on_left, lhs) != types_of_fields(on_right, rhs):
-        raise TypeError("Schema's of joining columns do not match")
+        raise ValueError(
+            "Can not Join.  No shared columns between %s and %s" % (lhs, rhs),
+        )
+    left_types = listpack(types_of_fields(on_left, lhs))
+    right_types = listpack(types_of_fields(on_right, rhs))
+    if len(left_types) != len(right_types):
+        raise ValueError(
+            'Length of on_left=%d not equal to length of on_right=%d' % (
+                len(left_types), len(right_types),
+            ),
+        )
+
+    for n, promotion in enumerate(map(partial(promote, promote_option=False),
+                                      left_types,
+                                      right_types)):
+        if promotion == object_:
+            raise TypeError(
+                'Schemata of joining columns do not match,'
+                ' no promotion found for %s=%s and %s=%s' % (
+                    on_left[n], left_types[n], on_right[n], right_types[n],
+                ),
+            )
     _on_left = tuple(on_left) if isinstance(on_left, list) else on_left
     _on_right = (tuple(on_right) if isinstance(on_right, list)
                  else on_right)
@@ -448,9 +575,6 @@ def join(lhs, rhs, on_left=None, on_right=None,
     return Join(lhs, rhs, _on_left, _on_right, how, suffixes)
 
 
-join.__doc__ = Join.__doc__
-
-
 class Concat(Expr):
 
     """ Stack tables on common columns
@@ -514,6 +638,7 @@ def _shape_add(a, b):
     return Fixed(a.val + b.val)
 
 
+@copydoc(Concat)
 def concat(lhs, rhs, axis=0):
     ldshape = lhs.dshape
     rdshape = rhs.dshape
@@ -543,9 +668,6 @@ def concat(lhs, rhs, axis=0):
     return Concat(lhs, rhs, axis)
 
 
-concat.__doc__ = Concat.__doc__
-
-
 class IsIn(ElemWise):
     """Check if an expression contains values from a set.
 
@@ -581,6 +703,7 @@ class IsIn(ElemWise):
                               self._keys)
 
 
+@copydoc(IsIn)
 def isin(expr, keys):
     if isinstance(keys, Expr):
         raise TypeError('keys argument cannot be an expression, '
@@ -589,11 +712,8 @@ def isin(expr, keys):
     return IsIn(expr, frozenset(keys))
 
 
-isin.__doc__ = IsIn.__doc__
-
-
 dshape_method_list.extend([
-    (iscollection, set([sort, head])),
+    (iscollection, set([sort, head, tail])),
     (lambda ds: len(ds.shape) == 1, set([distinct])),
     (lambda ds: len(ds.shape) == 1 and isscalar(ds.measure), set([isin])),
 ])
