@@ -1,10 +1,13 @@
 from __future__ import absolute_import, division, print_function
 
-import flask
-import requests
+import warnings
 
-from odo import resource
 from datashape import dshape
+import flask
+from flask.testing import FlaskClient
+from odo import resource
+import requests
+from requests.packages.urllib3.exceptions import InsecureRequestWarning
 
 from ..expr import Expr
 from ..dispatch import dispatch
@@ -40,6 +43,31 @@ def reason(response):
         return response.text
 
 
+def _request(method, client, url, params=None, auth=None, **kwargs):
+    if not isinstance(requests, FlaskClient):
+        kwargs['verify'] = client.verify_ssl
+        kwargs['params'] = params
+        kwargs['auth'] = auth
+
+    with warnings.catch_warnings():
+        warnings.simplefilter('ignore', InsecureRequestWarning)
+        return method(
+            '{base}{url}'.format(
+                base=client.url,
+                url=url,
+            ),
+            **kwargs
+        )
+
+
+def get(*args, **kwargs):
+    return _request(requests.get, *args, **kwargs)
+
+
+def post(*args, **kwargs):
+    return _request(requests.post, *args, **kwargs)
+
+
 class Client(object):
 
     """ Client for Blaze Server
@@ -51,11 +79,16 @@ class Client(object):
 
     url : str
         URL of a Blaze server
-
     serial : SerializationFormat, optional
         The serialization format object to use. Defaults to JSON.
         A serialization format is an object that supports:
         name, loads, and dumps.
+    verify_ssl : bool, optional
+        Verify the ssl certificate from the server.
+        This is enabled by default.
+    auth : tuple, optional
+        The username and password to use when connecting to the server.
+        If not provided, no auth header will be sent.
 
     Examples
     --------
@@ -70,20 +103,21 @@ class Client(object):
 
     blaze.server.server.Server
     """
-    __slots__ = 'url', 'serial'
+    __slots__ = 'url', 'serial', 'verify_ssl', 'auth'
 
-    def __init__(self, url, serial=json, **kwargs):
+    def __init__(self, url, serial=json, verify_ssl=True, auth=None, **kwargs):
         url = url.strip('/')
-        if not url[:4] == 'http':
+        if not url.startswith('http'):
             url = 'http://' + url
         self.url = url
         self.serial = serial
+        self.verify_ssl = verify_ssl
+        self.auth = auth
 
     @property
     def dshape(self):
         """The datashape of the client"""
-        response = requests.get('%s/datashape' % self.url)
-
+        response = get(self, '/datashape', auth=self.auth)
         if not ok(response):
             raise ValueError("Bad Response: %s" % reason(response))
 
@@ -95,14 +129,32 @@ def discover(c):
     return c.dshape
 
 
+def mimetype(serial):
+    """Function to generate a blaze serialization format mimetype put into a
+    dictionary of headers for consumption by requests.
+
+    Examples
+    --------
+    >>> from blaze.server.serialization import msgpack
+    >>> mimetype(msgpack)
+    {'Content-Type': 'application/vnd.blaze+msgpack'}
+    """
+    return {'Content-Type': 'application/vnd.blaze+%s' % serial.name}
+
+
 @dispatch(Expr, Client)
 def compute_down(expr, ec, **kwargs):
     from .server import to_tree
     tree = to_tree(expr)
 
     serial = ec.serial
-    r = requests.get('%s/compute.%s' % (ec.url, serial.name),
-                     data=serial.dumps({'expr': tree}))
+    r = post(
+        ec,
+        '/compute',
+        data=serial.dumps({'expr': tree}),
+        auth=ec.auth,
+        headers=mimetype(serial),
+    )
 
     if not ok(r):
         raise ValueError("Bad response: %s" % reason(r))
